"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const nock_1 = __importDefault(require("nock"));
const path_1 = __importDefault(require("path"));
const diff_1 = require("../diff");
const graphql_1 = require("graphql");
const introspectionResponse_json_1 = __importDefault(require("./fixtures/introspectionResponse.json"));
describe('getDiff', () => {
    describe('remote schema fetching', () => {
        const testRemoteSchemaLocation = 'http://test/graphql';
        const introspectionQuery = graphql_1.getIntrospectionQuery({ descriptions: false });
        it('fetches remote schema successfully', async () => {
            nock_1.default(testRemoteSchemaLocation)
                .post('', JSON.stringify({ query: introspectionQuery }))
                .twice()
                .reply(200, introspectionResponse_json_1.default);
            const result = await diff_1.getDiff(testRemoteSchemaLocation, testRemoteSchemaLocation);
            expect(result).toBeUndefined();
        });
        it('fetches remote schemas with headers', async () => {
            nock_1.default(testRemoteSchemaLocation)
                .matchHeader('test', 'test')
                .post('', JSON.stringify({ query: introspectionQuery }))
                .twice()
                .reply(200, introspectionResponse_json_1.default);
            const result = await diff_1.getDiff(testRemoteSchemaLocation, testRemoteSchemaLocation, {
                headers: {
                    Test: 'test'
                }
            });
            expect(result).toBeUndefined();
        });
        it('fetches remote schemas with left and right schema headers', async () => {
            const testRemoteRightSchemaLocation = 'http://testRight/graphql';
            nock_1.default(testRemoteSchemaLocation)
                .matchHeader('test', 'left')
                .post('', JSON.stringify({ query: introspectionQuery }))
                .reply(200, introspectionResponse_json_1.default);
            nock_1.default(testRemoteRightSchemaLocation)
                .matchHeader('test', 'right')
                .post('', JSON.stringify({ query: introspectionQuery }))
                .reply(200, introspectionResponse_json_1.default);
            const result = await diff_1.getDiff(testRemoteSchemaLocation, testRemoteRightSchemaLocation, {
                leftSchema: {
                    headers: {
                        Test: 'left'
                    }
                },
                rightSchema: {
                    headers: {
                        Test: 'right'
                    }
                }
            });
            expect(result).toBeUndefined();
        });
        it('fetches remote schemas with merged schema headers', async () => {
            const testRemoteRightSchemaLocation = 'http://testRight/graphql';
            nock_1.default(testRemoteSchemaLocation)
                .matchHeader('global', 'merged')
                .matchHeader('test', 'left')
                .post('', JSON.stringify({ query: introspectionQuery }))
                .reply(200, introspectionResponse_json_1.default);
            nock_1.default(testRemoteRightSchemaLocation)
                .matchHeader('global', 'merged')
                .matchHeader('test', 'right')
                .post('', JSON.stringify({ query: introspectionQuery }))
                .reply(200, introspectionResponse_json_1.default);
            const result = await diff_1.getDiff(testRemoteSchemaLocation, testRemoteRightSchemaLocation, {
                headers: {
                    Global: 'merged',
                },
                leftSchema: {
                    headers: {
                        Test: 'left'
                    }
                },
                rightSchema: {
                    headers: {
                        Test: 'right'
                    }
                }
            });
            expect(result).toBeUndefined();
        });
        it('throws error on status codes other than 200', () => {
            nock_1.default(testRemoteSchemaLocation)
                .post('', JSON.stringify({ query: introspectionQuery }))
                .twice()
                .reply(404);
            return expect(diff_1.getDiff(testRemoteSchemaLocation, testRemoteSchemaLocation)).rejects.toThrow(`404 - Not Found (${testRemoteSchemaLocation})`);
        });
        it('throws error on invalid response', () => {
            nock_1.default(testRemoteSchemaLocation)
                .post('', JSON.stringify({ query: introspectionQuery }))
                .twice()
                .reply(200, { invalid: 'response' });
            return expect(diff_1.getDiff(testRemoteSchemaLocation, testRemoteSchemaLocation)).rejects.toThrow(`Invalid response from GraphQL endpoint: ${testRemoteSchemaLocation}`);
        });
        afterEach(() => {
            nock_1.default.cleanAll();
        });
    });
    describe('local schema reading', () => {
        it('works with exact path to file', async () => {
            const localSchemaLocation = path_1.default.join(__dirname, './fixtures/localSchema.graphql');
            const result = await diff_1.getDiff(localSchemaLocation, localSchemaLocation);
            expect(result).toBeUndefined();
        });
        it('works with glob pattern', async () => {
            const localSchemaLocation = path_1.default.join(__dirname, './fixtures/**/localSchema.graphql');
            const result = await diff_1.getDiff(localSchemaLocation, localSchemaLocation);
            expect(result).toBeUndefined();
        });
        it('throws error on non-existent path', () => {
            return expect(diff_1.getDiff(path_1.default.join(__dirname, 'invalidLocation'), path_1.default.join(__dirname, 'invalidLocation'))).rejects.toThrow(/ENOENT/);
        });
        it('throws error on non-existent files in glob pattern', () => {
            return expect(diff_1.getDiff(path_1.default.join(__dirname, '/**/*.invalidgql'), path_1.default.join(__dirname, '/**/*.invalidgql'))).rejects.toThrow(/No types found with glob pattern '.*'/);
        });
    });
    describe('schema diffing', () => {
        it('returns the exact diff between two schemas', async () => {
            const result = await diff_1.getDiff(path_1.default.join(__dirname, 'fixtures/localSchema.graphql'), path_1.default.join(__dirname, 'fixtures/localSchemaDangerous.graphql'));
            expect(result).toBeDefined();
            if (result) {
                expect(result.diff).toMatch(/\+  SECOND_VALUE/);
            }
        });
        it('returns dangerous changes', async () => {
            const result = await diff_1.getDiff(path_1.default.join(__dirname, 'fixtures/localSchema.graphql'), path_1.default.join(__dirname, 'fixtures/localSchemaDangerous.graphql'));
            expect(result).toBeDefined();
            if (result) {
                expect(result.dangerousChanges).toEqual([
                    {
                        description: 'SECOND_VALUE was added to enum type TestEnum.',
                        type: 'VALUE_ADDED_TO_ENUM'
                    }
                ]);
            }
        });
        it('returns breaking changes', async () => {
            const result = await diff_1.getDiff(path_1.default.join(__dirname, 'fixtures/localSchema.graphql'), path_1.default.join(__dirname, 'fixtures/localSchemaBreaking.graphql'));
            expect(result).toBeDefined();
            if (result) {
                expect(result.breakingChanges).toEqual([
                    {
                        description: 'Query.test changed type from String to Int.',
                        type: 'FIELD_CHANGED_KIND'
                    }
                ]);
            }
        });
    });
    describe('schema sorting', () => {
        it('returns diff between two unsorted, but otherwise equal schemas, when sorting not enabled', async () => {
            const result = await diff_1.getDiff(path_1.default.join(__dirname, 'fixtures/localSchemaSorted.graphql'), path_1.default.join(__dirname, 'fixtures/localSchemaUnsorted.graphql'));
            expect(result).toBeDefined();
        });
        it('returns nothing between two unsorted, but otherwise equal schemas, when sorting enabled', async () => {
            const result = await diff_1.getDiff(path_1.default.join(__dirname, 'fixtures/localSchemaSorted.graphql'), path_1.default.join(__dirname, 'fixtures/localSchemaUnsorted.graphql'), { sortSchema: true });
            expect(result).toBeUndefined();
        });
    });
});
