"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const utilities_1 = require("graphql/utilities");
const fs_1 = __importDefault(require("fs"));
const is_glob_1 = __importDefault(require("is-glob"));
const node_fetch_1 = __importDefault(require("node-fetch"));
const disparity_1 = __importDefault(require("disparity"));
const merge_graphql_schemas_1 = require("merge-graphql-schemas");
async function fetchRemoteSchema(endpoint, headers) {
    const introspectionQuery = graphql_1.getIntrospectionQuery({ descriptions: false });
    const res = await node_fetch_1.default(endpoint, {
        method: 'POST',
        headers: {
            'content-type': 'application/json',
            ...headers
        },
        body: JSON.stringify({ query: introspectionQuery })
    });
    if (!res.ok) {
        throw new Error(`${res.status} - ${res.statusText} (${endpoint})`);
    }
    const responseBody = await res.json();
    if (!responseBody || !responseBody.data || !responseBody.data.__schema) {
        throw new Error(`Invalid response from GraphQL endpoint: ${endpoint}`);
    }
    return graphql_1.buildClientSchema(responseBody.data);
}
function readLocalSchema(schemaPath) {
    let schemaString;
    if (is_glob_1.default(schemaPath)) {
        const typesArray = merge_graphql_schemas_1.fileLoader(schemaPath);
        if (typesArray.length === 0) {
            throw new Error(`No types found with glob pattern '${schemaPath}'`);
        }
        schemaString = merge_graphql_schemas_1.mergeTypes(typesArray, { all: true });
    }
    else {
        schemaString = fs_1.default.readFileSync(schemaPath, 'utf8');
    }
    const schema = graphql_1.buildSchema(schemaString);
    const introspection = graphql_1.introspectionFromSchema(schema, {
        descriptions: false
    });
    return graphql_1.buildClientSchema(introspection);
}
async function getSchema(schemaLocation, options = {}) {
    if (schemaLocation.match(/^https?/)) {
        return fetchRemoteSchema(schemaLocation, options.headers);
    }
    else {
        return readLocalSchema(schemaLocation);
    }
}
async function getDiff(leftSchemaLocation, rightSchemaLocation, options = {}) {
    const leftSchemaOptions = {
        headers: {
            ...options.headers,
            ...(options.leftSchema && options.leftSchema.headers)
        }
    };
    const rightSchemaOptions = {
        headers: {
            ...options.headers,
            ...(options.rightSchema && options.rightSchema.headers)
        }
    };
    let [leftSchema, rightSchema] = await Promise.all([
        getSchema(leftSchemaLocation, leftSchemaOptions),
        getSchema(rightSchemaLocation, rightSchemaOptions)
    ]);
    if (!leftSchema || !rightSchema) {
        throw new Error('Schemas not defined');
    }
    if (options.sortSchema) {
        [leftSchema, rightSchema] = [
            utilities_1.lexicographicSortSchema(leftSchema),
            utilities_1.lexicographicSortSchema(rightSchema)
        ];
    }
    const [leftSchemaSDL, rightSchemaSDL] = [
        graphql_1.printSchema(leftSchema),
        graphql_1.printSchema(rightSchema)
    ];
    if (leftSchemaSDL === rightSchemaSDL) {
        return;
    }
    const diff = disparity_1.default.unified(leftSchemaSDL, rightSchemaSDL, {
        paths: [leftSchemaLocation, rightSchemaLocation]
    });
    const diffNoColor = disparity_1.default.unifiedNoColor(leftSchemaSDL, rightSchemaSDL, {
        paths: [leftSchemaLocation, rightSchemaLocation]
    });
    const dangerousChanges = graphql_1.findDangerousChanges(leftSchema, rightSchema);
    const breakingChanges = graphql_1.findBreakingChanges(leftSchema, rightSchema);
    return {
        diff,
        diffNoColor,
        dangerousChanges,
        breakingChanges
    };
}
exports.getDiff = getDiff;
